@@ -1,3 +1,5 @@
+# frozen-string-literal: true
+
 Sequel.require 'adapters/shared/mssql'
 
 module Sequel
@@ -7,7 +9,6 @@ module Sequel
     module MSSQL
       # Database instance methods for MSSQL databases accessed via JDBC.
       module DatabaseMethods
-        extend Sequel::Database::ResetIdentifierMangling
         PRIMARY_KEY_INDEX_RE = /\Apk__/i.freeze
         ATAT_IDENTITY = 'SELECT @@IDENTITY'.freeze
         SCOPE_IDENTITY = 'SELECT SCOPE_IDENTITY()'.freeze
@@ -20,7 +21,7 @@ module Sequel
         def last_insert_id(conn, opts=OPTS)
           statement(conn) do |stmt|
             sql = opts[:prepared] ? ATAT_IDENTITY : SCOPE_IDENTITY
-            rs = log_yield(sql){stmt.executeQuery(sql)}
+            rs = log_connection_yield(sql, conn){stmt.executeQuery(sql)}
             rs.next
             rs.getLong(1)
           end
