@@ -1,3 +1,5 @@
+# frozen-string-literal: true
+
 module Sequel
   # The Schema module holds the schema generators.
   module Schema
@@ -16,8 +18,7 @@ module Sequel
     # the {"Schema Modification" guide}[rdoc-ref:doc/schema_modification.rdoc].
     class CreateTableGenerator
       # Classes specifying generic types that Sequel will convert to database-specific types.
-      GENERIC_TYPES=[String, Integer, Fixnum, Bignum, Float, Numeric, BigDecimal,
-      Date, DateTime, Time, File, TrueClass, FalseClass]
+      GENERIC_TYPES=%w'String Integer Float Numeric BigDecimal Date DateTime Time File TrueClass FalseClass'.freeze
       
       # Return the column hashes created by this generator
       attr_reader :columns
@@ -39,7 +40,19 @@ module Sequel
         instance_eval(&block) if block
         @columns.unshift(@primary_key) if @primary_key && !has_column?(primary_key_name)
       end
+
+      # Use custom Bignum method to use :Bignum instead of Bignum class, to work
+      # correctly in cases where Bignum is the same as Integer.
+      def Bignum(name, opts=OPTS)
+        column(name, :Bignum, opts)
+      end
       
+      # Use custom Fixnum method to use Integer instead of Fixnum class, to avoid 
+      # warnings on ruby 2.4+.
+      def Fixnum(name, opts=OPTS)
+        column(name, Integer, opts)
+      end
+
       # Add a method for each of the given types that creates a column
       # with that type as a constant.  Types given should either already
       # be constants/classes or a capitalized string/symbol with the same name
@@ -81,6 +94,10 @@ module Sequel
       #
       # The following options are supported:
       #
+      # :collate :: The collation to use for the column.  For backwards compatibility,
+      #             only symbols and string values are supported, and they are used verbatim.
+      #             However, on PostgreSQL, symbols are literalized as regular identifiers,
+      #             since unquoted collations are unlikely to be valid.
       # :default :: The default value for the column.
       # :deferrable :: For foreign key columns, this ensures referential integrity will work even if
       #                referencing table uses a foreign key value that does not
@@ -150,12 +167,10 @@ module Sequel
         opts = case table
         when Hash
           table.merge(opts)
-        when Symbol
-          opts.merge(:table=>table)
         when NilClass
           opts
         else
-          raise(Error, "The second argument to foreign_key should be a Hash, Symbol, or nil")
+          opts.merge(:table=>table)
         end
         return composite_foreign_key(name, opts) if name.is_a?(Array)
         column(name, Integer, opts)
@@ -232,22 +247,35 @@ module Sequel
       #
       # If an array of column symbols is used, you can specify the :name option
       # to name the constraint.
+      #
+      # Options:
+      # :keep_order :: For non-composite primary keys, respects the existing order of
+      #                columns, overriding the default behavior of making the primary
+      #                key the first column.
       # 
       # Examples:
       #   primary_key(:id)
+      #   primary_key(:id, Bigint)
+      #   primary_key(:id, Bigint, :keep_order=>true)
       #   primary_key([:street_number, :house_number], :name=>:some constraint_name)
       def primary_key(name, *args)
         return composite_primary_key(name, *args) if name.is_a?(Array)
-        @primary_key = @db.serial_primary_key_options.merge({:name => name})
+        column = @db.serial_primary_key_options.merge({:name => name})
         
         if opts = args.pop
           opts = {:type => opts} unless opts.is_a?(Hash)
           if type = args.pop
-            opts.merge!(:type => type)
+            opts = opts.merge(:type => type)
           end
-          @primary_key.merge!(opts)
+          column.merge!(opts)
+        end
+
+        @primary_key = column
+        if column[:keep_order]
+          columns << column
+        else
+          columns.unshift(column)
         end
-        @primary_key
       end
 
       # The name of the primary key for this generator, if it has a primary key.
@@ -314,6 +342,15 @@ module Sequel
       # See CreateTableGenerator#column for the available options.
       #
       #   add_column(:name, String) # ADD COLUMN name varchar(255)
+      #
+      # PostgreSQL specific options:
+      #
+      # :if_not_exists :: Set to true to not add the column if it already exists (PostgreSQL 9.6+)
+      #
+      # MySQL specific options:
+      #
+      # :after :: The name of an existing column that the new column should be positioned after
+      # :first :: Create this new column before all other existing columns
       def add_column(name, type, opts = OPTS)
         @operations << {:op => :add_column, :name => name, :type => type}.merge!(opts)
       end
@@ -377,6 +414,33 @@ module Sequel
       # CreateTableGenerator#index for available options.
       #
       #   add_index(:artist_id) # CREATE INDEX table_artist_id_index ON table (artist_id)
+      #
+      # Options:
+      #
+      # :name :: Give a specific name for the index. Highly recommended if you plan on
+      #          dropping the index later.
+      # :where :: A filter expression, used to setup a partial index (if supported).
+      # :unique :: Create a unique index.
+      #
+      # PostgreSQL specific options:
+      #
+      # :concurrently :: Create the index concurrently, so it doesn't require an exclusive lock
+      #                  on the table.
+      # :index_type :: The underlying index type to use for a full_text index, gin by default).
+      # :language :: The language to use for a full text index (simple by default).
+      # :opclass :: Set an opclass to use for all columns (per-column opclasses require
+      #             custom SQL).
+      # :type :: Set the index type (e.g. full_text, spatial, hash, gin, gist, btree).
+      #
+      # MySQL specific options:
+      #
+      # :type :: Set the index type, with full_text and spatial indexes handled specially.
+      #
+      # Microsoft SQL Server specific options:
+      #
+      # :include :: Includes additional columns in the index.
+      # :key_index :: Sets the KEY INDEX to the given value.
+      # :type :: clustered uses a clustered index, full_text uses a full text index.
       def add_index(columns, opts = OPTS)
         @operations << {:op => :add_index, :columns => Array(columns)}.merge!(opts)
       end
@@ -403,6 +467,15 @@ module Sequel
       #
       #   drop_column(:artist_id) # DROP COLUMN artist_id
       #   drop_column(:artist_id, :cascade=>true) # DROP COLUMN artist_id CASCADE
+      #
+      # Options:
+      #
+      # :cascade :: CASCADE the operation, dropping other objects that depend on
+      #             the dropped column.
+      # 
+      # PostgreSQL specific options:
+      # :if_exists :: Use IF EXISTS, so no error is raised if the column does not
+      #               exist.
       def drop_column(name, opts=OPTS)
         @operations << {:op => :drop_column, :name => name}.merge!(opts)
       end
@@ -463,6 +536,10 @@ module Sequel
       #
       #   set_column_default(:artist_name, 'a') # ALTER COLUMN artist_name SET DEFAULT 'a'
       #
+      # To remove an existing default value, use +nil+ as the value:
+      #
+      #   set_column_default(:artist_name, nil) # ALTER COLUMN artist_name SET DEFAULT NULL
+      #
       # On MySQL, make sure to use a symbol for the name of the column, as otherwise you
       # can lose the type and NULL/NOT NULL setting for the column.
       def set_column_default(name, default)
