@@ -1,3 +1,5 @@
+# frozen-string-literal: true
+
 module Sequel
   class Dataset
     # ---------------------
@@ -16,23 +18,23 @@ module Sequel
 
     # Which options don't affect the SQL generation.  Used by simple_select_all?
     # to determine if this is a simple SELECT * FROM table.
-    NON_SQL_OPTIONS = [:server, :defaults, :overrides, :graph, :eager, :eager_graph, :graph_aliases]
+    NON_SQL_OPTIONS = [:server, :graph, :eager, :eager_graph, :graph_aliases, :row_proc, :quote_identifiers, :identifier_input_method, :identifier_output_method, :skip_symbol_cache, :model, :model_object, :association_reflection, :fetch, :numrows, :autoid].freeze
     
     # These symbols have _join methods created (e.g. inner_join) that
     # call join_table with the symbol, passing along the arguments and
     # block from the method call.
-    CONDITIONED_JOIN_TYPES = [:inner, :full_outer, :right_outer, :left_outer, :full, :right, :left]
+    CONDITIONED_JOIN_TYPES = [:inner, :full_outer, :right_outer, :left_outer, :full, :right, :left].freeze
 
     # These symbols have _join methods created (e.g. natural_join).
     # They accept a table argument and options hash which is passed to join_table,
     # and they raise an error if called with a block.
-    UNCONDITIONED_JOIN_TYPES = [:natural, :natural_left, :natural_right, :natural_full, :cross]
+    UNCONDITIONED_JOIN_TYPES = [:natural, :natural_left, :natural_right, :natural_full, :cross].freeze
     
     # All methods that return modified datasets with a joined table added.
-    JOIN_METHODS = (CONDITIONED_JOIN_TYPES + UNCONDITIONED_JOIN_TYPES).map{|x| "#{x}_join".to_sym} + [:join, :join_table]
+    JOIN_METHODS = ((CONDITIONED_JOIN_TYPES + UNCONDITIONED_JOIN_TYPES).map{|x| "#{x}_join".to_sym} + [:join, :join_table]).freeze
     
     # Methods that return modified datasets
-    QUERY_METHODS = (<<-METHS).split.map(&:to_sym) + JOIN_METHODS
+    QUERY_METHODS = ((<<-METHS).split.map(&:to_sym) + JOIN_METHODS).freeze
       add_graph_aliases and distinct except exclude exclude_having exclude_where
       filter for_update from from_self graph grep group group_and_count group_append group_by having intersect invert
       limit lock_style naked offset or order order_append order_by order_more order_prepend qualify
@@ -67,19 +69,43 @@ module Sequel
       where(*cond, &block)
     end
     
-    # Returns a new clone of the dataset with the given options merged.
-    # If the options changed include options in COLUMN_CHANGE_OPTS, the cached
-    # columns are deleted.  This method should generally not be called
-    # directly by user code.
-    def clone(opts = nil)
-      c = super()
-      if opts
-        c.instance_variable_set(:@opts, Hash[@opts].merge!(opts))
-        c.instance_variable_set(:@columns, nil) if @columns && !opts.each_key{|o| break if COLUMN_CHANGE_OPTS.include?(o)}
-      else
-        c.instance_variable_set(:@opts, Hash[@opts])
+    # On Ruby 2.4+, use clone(:freeze=>false) to create clones, because
+    # we use true freezing in that case, and we need to modify the opts
+    # in the frozen copy.
+    #
+    # On Ruby <2.4, just use Object#clone directly, since we don't
+    # use true freezing as it isn't possible.
+    if TRUE_FREEZE
+      # Save original clone implementation, as some other methods need
+      # to call it internally.
+      alias _clone clone
+      private :_clone
+
+      # Returns a new clone of the dataset with the given options merged.
+      # If the options changed include options in COLUMN_CHANGE_OPTS, the cached
+      # columns are deleted.  This method should generally not be called
+      # directly by user code.
+      def clone(opts = OPTS)
+        c = super(:freeze=>false)
+        c.opts.merge!(opts)
+        unless opts.each_key{|o| break if COLUMN_CHANGE_OPTS.include?(o)}
+          c.clear_columns_cache
+        end
+        c.freeze if frozen?
+        c
       end
-      c
+    else
+      # :nocov:
+      def clone(opts = OPTS) # :nodoc:
+        c = super()
+        c.opts.merge!(opts)
+        unless opts.each_key{|o| break if COLUMN_CHANGE_OPTS.include?(o)}
+          c.clear_columns_cache
+        end
+        c.freeze if frozen?
+        c
+      end
+      # :nocov:
     end
 
     # Returns a copy of the dataset with the SQL DISTINCT clause. The DISTINCT
@@ -95,7 +121,7 @@ module Sequel
     def distinct(*args, &block)
       virtual_row_columns(args, block)
       raise(InvalidOperation, "DISTINCT ON not supported") if !args.empty? && !supports_distinct_on?
-      clone(:distinct => args)
+      clone(:distinct => args.freeze)
     end
 
     # Adds an EXCEPT clause using a second dataset object.
@@ -129,6 +155,22 @@ module Sequel
     #   
     #   DB[:items].exclude(:category => 'software', :id=>3)
     #   # SELECT * FROM items WHERE ((category != 'software') OR (id != 3))
+    #
+    # Also note that SQL uses 3-valued boolean logic (+true+, +false+, +NULL+), so
+    # the inverse of a true condition is a false condition, and will still
+    # not match rows that were NULL originally.  If you take the earlier
+    # example:
+    #
+    #   DB[:items].exclude(:category => 'software')
+    #   # SELECT * FROM items WHERE (category != 'software')
+    #
+    # Note that this does not match rows where +category+ is +NULL+.  This
+    # is because +NULL+ is an unknown value, and you do not know whether
+    # or not the +NULL+ category is +software+.  You can explicitly
+    # specify how to handle +NULL+ values if you want:
+    #
+    #   DB[:items].exclude(Sequel.~(:category=>nil) & {:category => 'software'})
+    #   # SELECT * FROM items WHERE ((category IS NULL) OR (category != 'software'))
     def exclude(*cond, &block)
       _filter_or_exclude(true, :where, *cond, &block)
     end
@@ -137,6 +179,9 @@ module Sequel
     #
     #   DB[:items].select_group(:name).exclude_having{count(name) < 2}
     #   # SELECT name FROM items GROUP BY name HAVING (count(name) >= 2)
+    #
+    # See documentation for exclude for how inversion is handled in regards
+    # to SQL 3-valued boolean logic.
     def exclude_having(*cond, &block)
       _filter_or_exclude(true, :having, *cond, &block)
     end
@@ -146,9 +191,24 @@ module Sequel
       exclude(*cond, &block)
     end
 
-    # Return a clone of the dataset loaded with the extensions, see #extension!.
-    def extension(*exts)
-      clone.extension!(*exts)
+    if TRUE_FREEZE
+      # Return a clone of the dataset loaded with the given dataset extensions.
+      # If no related extension file exists or the extension does not have
+      # specific support for Dataset objects, an Error will be raised.
+      def extension(*a)
+        c = _clone(:freeze=>false)
+        c.send(:_extension!, a)
+        c.freeze if frozen?
+        c
+      end
+    else
+      # :nocov:
+      def extension(*exts) # :nodoc:
+        c = clone
+        c.send(:_extension!, exts)
+        c
+      end
+      # :nocov:
     end
 
     # Alias for where.
@@ -160,7 +220,7 @@ module Sequel
     #
     #   DB[:table].for_update # SELECT * FROM table FOR UPDATE
     def for_update
-      lock_style(:update)
+      cached_dataset(:_for_update_ds){lock_style(:update)}
     end
 
     # Returns a copy of the dataset with the source changed. If no
@@ -197,8 +257,8 @@ module Sequel
           s
         end
       end
-      o = {:from=>source.empty? ? nil : source}
-      o[:with] = (opts[:with] || []) + ctes if ctes
+      o = {:from=>source.empty? ? nil : source.freeze}
+      o[:with] = ((opts[:with] || []) + ctes).freeze if ctes
       o[:num_dataset_sources] = table_alias_num if table_alias_num > 0
       clone(o)
     end
@@ -219,8 +279,13 @@ module Sequel
     #   # SELECT * FROM (SELECT id, name FROM items ORDER BY name) AS foo(c1, c2)
     def from_self(opts=OPTS)
       fs = {}
-      @opts.keys.each{|k| fs[k] = nil unless NON_SQL_OPTIONS.include?(k)}
-      clone(fs).from(opts[:alias] ? as(opts[:alias], opts[:column_aliases]) : self)
+      non_sql = non_sql_options
+      @opts.keys.each{|k| fs[k] = nil unless non_sql.include?(k)}
+      c = clone(fs).from(opts[:alias] ? as(opts[:alias], opts[:column_aliases]) : self)
+      if cols = _columns
+        c.send(:columns=, cols)
+      end
+      c
     end
 
     # Match any of the columns to any of the patterns. The terms can be
@@ -281,7 +346,7 @@ module Sequel
     #   DB[:items].group{[a, sum(b)]} # SELECT * FROM items GROUP BY a, sum(b)
     def group(*columns, &block)
       virtual_row_columns(columns, block)
-      clone(:group => (columns.compact.empty? ? nil : columns))
+      clone(:group => (columns.compact.empty? ? nil : columns.freeze))
     end
 
     # Alias of group
@@ -311,7 +376,7 @@ module Sequel
     #   # SELECT substr(first_name, 1, 1) AS initial, count(*) AS count FROM items GROUP BY substr(first_name, 1, 1)
     #   # => [{:initial=>'a', :count=>1}, ...]
     def group_and_count(*columns, &block)
-      select_group(*columns, &block).select_more(COUNT_OF_ALL_AS_COUNT)
+      select_group(*columns, &block).select_append(COUNT_OF_ALL_AS_COUNT)
     end
 
     # Returns a copy of the dataset with the given columns added to the list of
@@ -382,15 +447,20 @@ module Sequel
     #
     #   DB[:items].where(:category => 'software', :id=>3).invert
     #   # SELECT * FROM items WHERE ((category != 'software') OR (id != 3))
+    #
+    # See documentation for exclude for how inversion is handled in regards
+    # to SQL 3-valued boolean logic.
     def invert
-      having, where = @opts.values_at(:having, :where)
-      if having.nil? && where.nil?
-        where(false)
-      else
-        o = {}
-        o[:having] = SQL::BooleanExpression.invert(having) if having
-        o[:where] = SQL::BooleanExpression.invert(where) if where
-        clone(o)
+      cached_dataset(:_invert_ds) do
+        having, where = @opts.values_at(:having, :where)
+        if having.nil? && where.nil?
+          where(false)
+        else
+          o = {}
+          o[:having] = SQL::BooleanExpression.invert(having) if having
+          o[:where] = SQL::BooleanExpression.invert(where) if where
+          clone(o)
+        end
       end
     end
 
@@ -528,7 +598,7 @@ module Sequel
         SQL::JoinOnClause.new(expr, type, table_expr)
       end
 
-      opts = {:join => (@opts[:join] || []) + [join]}
+      opts = {:join => ((@opts[:join] || []) + [join]).freeze}
       opts[:last_joined_table] = table_name unless options[:reset_implicit_qualifier] == false
       opts[:num_dataset_sources] = table_alias_num if table_alias_num
       clone(opts)
@@ -608,9 +678,7 @@ module Sequel
     #   ds.all # => [{2=>:id}]
     #   ds.naked.all # => [{:id=>2}]
     def naked
-      ds = clone
-      ds.row_proc = nil
-      ds
+      cached_dataset(:_naked_ds){with_row_proc(nil)}
     end
 
     # Returns a copy of the dataset with a specified order. Can be safely combined with limit.
@@ -656,12 +724,17 @@ module Sequel
     #   DB[:items].order(nil) # SELECT * FROM items
     def order(*columns, &block)
       virtual_row_columns(columns, block)
-      clone(:order => (columns.compact.empty?) ? nil : columns)
+      clone(:order => (columns.compact.empty?) ? nil : columns.freeze)
     end
     
-    # Alias of order_more, for naming consistency with order_prepend.
+    # Returns a copy of the dataset with the order columns added
+    # to the end of the existing order.
+    #
+    #   DB[:items].order(:a).order(:b) # SELECT * FROM items ORDER BY b
+    #   DB[:items].order(:a).order_append(:b) # SELECT * FROM items ORDER BY a, b
     def order_append(*columns, &block)
-      order_more(*columns, &block)
+      columns = @opts[:order] + columns if @opts[:order]
+      order(*columns, &block)
     end
 
     # Alias of order
@@ -669,14 +742,9 @@ module Sequel
       order(*columns, &block)
     end
 
-    # Returns a copy of the dataset with the order columns added
-    # to the end of the existing order.
-    #
-    #   DB[:items].order(:a).order(:b) # SELECT * FROM items ORDER BY b
-    #   DB[:items].order(:a).order_more(:b) # SELECT * FROM items ORDER BY a, b
+    # Alias of order_append.
     def order_more(*columns, &block)
-      columns = @opts[:order] + columns if @opts[:order]
-      order(*columns, &block)
+      order_append(*columns, &block)
     end
     
     # Returns a copy of the dataset with the order columns added
@@ -686,7 +754,7 @@ module Sequel
     #   DB[:items].order(:a).order_prepend(:b) # SELECT * FROM items ORDER BY b, a
     def order_prepend(*columns, &block)
       ds = order(*columns, &block)
-      @opts[:order] ? ds.order_more(*@opts[:order]) : ds
+      @opts[:order] ? ds.order_append(*@opts[:order]) : ds
     end
     
     # Qualify to the given table, or first source if no table is given.
@@ -703,7 +771,7 @@ module Sequel
       (o.keys & QUALIFY_KEYS).each do |k|
         h[k] = qualified_expression(o[k], table)
       end
-      h[:select] = [SQL::ColumnAll.new(table)] if !o[:select] || o[:select].empty?
+      h[:select] = [SQL::ColumnAll.new(table)].freeze if !o[:select] || o[:select].empty?
       clone(h)
     end
 
@@ -717,7 +785,7 @@ module Sequel
     #   DB[:items].returning(:id, :name) # RETURNING id, name
     def returning(*values)
       raise Error, "RETURNING is not supported on #{db.database_type}" unless supports_returning?(:insert)
-      clone(:returning=>values)
+      clone(:returning=>values.freeze)
     end
 
     # Returns a copy of the dataset with the order reversed. If no order is
@@ -728,8 +796,12 @@ module Sequel
     #   DB[:items].order(:id).reverse # SELECT * FROM items ORDER BY id DESC
     #   DB[:items].order(:id).reverse(Sequel.desc(:name)) # SELECT * FROM items ORDER BY name ASC
     def reverse(*order, &block)
-      virtual_row_columns(order, block)
-      order(*invert_order(order.empty? ? @opts[:order] : order))
+      if order.empty? && !block
+        cached_dataset(:_reverse_ds){order(*invert_order(@opts[:order]))}
+      else
+        virtual_row_columns(order, block)
+        order(*invert_order(order.empty? ? @opts[:order] : order.freeze))
+      end
     end
 
     # Alias of +reverse+
@@ -746,7 +818,7 @@ module Sequel
     #   DB[:items].select{[a, sum(b)]} # SELECT a, sum(b) FROM items
     def select(*columns, &block)
       virtual_row_columns(columns, block)
-      clone(:select => columns)
+      clone(:select => columns.freeze)
     end
     
     # Returns a copy of the dataset selecting the wildcard if no arguments
@@ -760,7 +832,7 @@ module Sequel
       if tables.empty?
         clone(:select => nil)
       else
-        select(*tables.map{|t| i, a = split_alias(t); a || i}.map{|t| SQL::ColumnAll.new(t)})
+        select(*tables.map{|t| i, a = split_alias(t); a || i}.map!{|t| SQL::ColumnAll.new(t)}.freeze)
       end
     end
     
@@ -775,7 +847,7 @@ module Sequel
       cur_sel = @opts[:select]
       if !cur_sel || cur_sel.empty?
         unless supports_select_all_and_column?
-          return select_all(*(Array(@opts[:from]) + Array(@opts[:join]))).select_more(*columns, &block)
+          return select_all(*(Array(@opts[:from]) + Array(@opts[:join]))).select_append(*columns, &block)
         end
         cur_sel = [WILDCARD]
       end
@@ -825,6 +897,14 @@ module Sequel
       end
     end
 
+    # Skip locked rows when returning results from this dataset.
+    def skip_locked
+      cached_dataset(:_skip_locked_ds) do
+        raise(Error, 'This dataset does not support skipping locked rows') unless supports_skip_locked?
+        clone(:skip_locked=>true)
+      end
+    end
+
     # Unbind bound variables from this dataset's filter and return an array of two
     # objects.  The first object is a modified dataset where the filter has been
     # replaced with one that uses bound variable placeholders.  The second object
@@ -846,7 +926,7 @@ module Sequel
     #   DB[:items].group(:a).having(:a=>1).where(:b).unfiltered
     #   # SELECT * FROM items GROUP BY a
     def unfiltered
-      clone(:where => nil, :having => nil)
+      cached_dataset(:_unfiltered_ds){clone(:where => nil, :having => nil)}
     end
 
     # Returns a copy of the dataset with no grouping (GROUP or HAVING clause) applied.
@@ -854,7 +934,7 @@ module Sequel
     #   DB[:items].group(:a).having(:a=>1).where(:b).ungrouped
     #   # SELECT * FROM items WHERE b
     def ungrouped
-      clone(:group => nil, :having => nil)
+      cached_dataset(:_ungrouped_ds){clone(:group => nil, :having => nil)}
     end
 
     # Adds a UNION clause using a second dataset object.
@@ -881,14 +961,14 @@ module Sequel
     # 
     #   DB[:items].limit(10, 20).unlimited # SELECT * FROM items
     def unlimited
-      clone(:limit=>nil, :offset=>nil)
+      cached_dataset(:_unlimited_ds){clone(:limit=>nil, :offset=>nil)}
     end
 
     # Returns a copy of the dataset with no order.
     # 
     #   DB[:items].order(:a).unordered # SELECT * FROM items
     def unordered
-      order(nil)
+      cached_dataset(:_unordered_ds){clone(:order=>nil)}
     end
     
     # Returns a copy of the dataset with the given WHERE conditions imposed upon it.  
@@ -959,7 +1039,7 @@ module Sequel
         s, ds = hoist_cte(dataset)
         s.with(name, ds, opts)
       else
-        clone(:with=>(@opts[:with]||[]) + [Hash[opts].merge!(:name=>name, :dataset=>dataset)])
+        clone(:with=>((@opts[:with]||[]) + [Hash[opts].merge!(:name=>name, :dataset=>dataset)]).freeze)
       end
     end
 
@@ -988,10 +1068,43 @@ module Sequel
         s, ds = hoist_cte(recursive)
         s.with_recursive(name, nonrecursive, ds, opts)
       else
-        clone(:with=>(@opts[:with]||[]) + [Hash[opts].merge!(:recursive=>true, :name=>name, :dataset=>nonrecursive.union(recursive, {:all=>opts[:union_all] != false, :from_self=>false}))])
+        clone(:with=>((@opts[:with]||[]) + [Hash[opts].merge!(:recursive=>true, :name=>name, :dataset=>nonrecursive.union(recursive, {:all=>opts[:union_all] != false, :from_self=>false}))]).freeze)
       end
     end
     
+    if TRUE_FREEZE
+      # Return a clone of the dataset extended with the given modules.
+      # Note that like Object#extend, when multiple modules are provided
+      # as arguments the cloned dataset is extended with the modules in reverse
+      # order.  If a block is provided, a module is created using the block and
+      # the clone is extended with that module after any modules given as arguments.
+      def with_extend(*mods, &block)
+        c = _clone(:freeze=>false)
+        c.extend(*mods) unless mods.empty?
+        c.extend(Module.new(&block)) if block
+        c.freeze if frozen?
+        c
+      end
+    else
+      # :nocov:
+      def with_extend(*mods, &block) # :nodoc:
+        c = clone
+        c.extend(*mods) unless mods.empty?
+        c.extend(Module.new(&block)) if block
+        c
+      end
+      # :nocov:
+    end
+
+    # Returns a cloned dataset with the given row_proc.
+    #
+    #   ds = DB[:items]
+    #   ds.all # => [{:id=>2}]
+    #   ds.with_row_proc(proc(&:invert)).all # => [{2=>:id}]
+    def with_row_proc(callable)
+      clone(:row_proc=>callable)
+    end
+
     # Returns a copy of the dataset with the static SQL used.  This is useful if you want
     # to keep the same row_proc/graph, but change the SQL used to custom SQL.
     #
@@ -1004,6 +1117,26 @@ module Sequel
     # You can also provide a method name and arguments to call to get the SQL:
     #
     #   DB[:items].with_sql(:insert_sql, :b=>1) # INSERT INTO items (b) VALUES (1)
+    #
+    # Note that datasets that specify custom SQL using this method will generally
+    # ignore future dataset methods that modify the SQL used, as specifying custom SQL
+    # overrides Sequel's SQL generator.  You should probably limit yourself to the following
+    # dataset methods when using this method:
+    #
+    # * each
+    # * all
+    # * single_record (if only one record could be returned)
+    # * single_value (if only one record could be returned, and a single column is selected)
+    # * map
+    # * to_hash
+    # * delete (if a DELETE statement)
+    # * update (if an UPDATE statement, with no arguments)
+    # * insert (if an INSERT statement, with no arguments)
+    # * truncate (if a TRUNCATE statement, with no arguments)
+    #
+    # If you want to use arbitrary dataset methods on a dataset that uses custom SQL, call
+    # from_self on the dataset, which wraps the custom SQL in a subquery, and allows normal
+    # dataset methods that modify the SQL to work.
     def with_sql(sql, *args)
       if sql.is_a?(Symbol)
         sql = send(sql, *args)
@@ -1021,7 +1154,7 @@ module Sequel
         s, ds = hoist_cte(dataset)
         return s.compound_clone(type, ds, opts)
       end
-      ds = compound_from_self.clone(:compounds=>Array(@opts[:compounds]).map(&:dup) + [[type, dataset.compound_from_self, opts[:all]]])
+      ds = compound_from_self.clone(:compounds=>(Array(@opts[:compounds]).map(&:dup) + [[type, dataset.compound_from_self, opts[:all]].freeze]).freeze)
       opts[:from_self] == false ? ds : ds.from_self(opts)
     end
 
@@ -1035,7 +1168,8 @@ module Sequel
     #   SELECT * FROM table
     #   SELECT table.* FROM table
     def simple_select_all?
-      o = @opts.reject{|k,v| v.nil? || NON_SQL_OPTIONS.include?(k)}
+      non_sql = non_sql_options
+      o = @opts.reject{|k,v| v.nil? || non_sql.include?(k)}
       if (f = o[:from]) && f.length == 1 && (f.first.is_a?(Symbol) || f.first.is_a?(SQL::AliasedExpression))
         case o.length
         when 1
@@ -1122,7 +1256,7 @@ module Sequel
     # clause from the given dataset added to it, and the second a clone of
     # the given dataset with the WITH clause removed.
     def hoist_cte(ds)
-      [clone(:with => (opts[:with] || []) + ds.opts[:with]), ds.clone(:with => nil)]
+      [clone(:with => ((opts[:with] || []) + ds.opts[:with]).freeze), ds.clone(:with => nil)]
     end
 
     # Whether CTEs need to be hoisted from the given ds into the current ds.
@@ -1153,6 +1287,11 @@ module Sequel
       server?(:default)
     end
 
+    # Dataset options that do not affect the generated SQL.
+    def non_sql_options
+      NON_SQL_OPTIONS
+    end
+
     # Treat the +block+ as a virtual_row block if not +nil+ and
     # add the resulting columns to the +columns+ array (modifies +columns+).
     def virtual_row_columns(columns, block)
