@@ -190,6 +190,24 @@ describe "SQLite temporary views" do
   end
 end
     
+describe "SQLite VALUES support" do
+  before do
+    @db = DB
+  end
+
+  it "should create a dataset using the VALUES clause via #values" do
+    @db.values([[1, 2], [3, 4]]).map([:column1, :column2]).must_equal [[1, 2], [3, 4]]
+  end
+
+  it "should support VALUES with unions" do
+    @db.values([[1]]).union(@db.values([[3]])).map(&:values).map(&:first).must_equal [1, 3]
+  end
+
+  it "should support VALUES in CTEs" do
+    @db[:a].cross_join(:b).with(:a, @db.values([[1, 2]]), :args=>[:c1, :c2]).with(:b, @db.values([[3, 4]]), :args=>[:c3, :c4]).map([:c1, :c2, :c3, :c4]).must_equal [[1, 2, 3, 4]]
+  end
+end if DB.sqlite_version >= 30803
+
 describe "SQLite type conversion" do
   before do
     @db = DB
@@ -279,7 +297,7 @@ end if DB.adapter_scheme == :sqlite
 
 describe "An SQLite dataset" do
   before do
-    @d = DB[:items]
+    @d = DB.dataset
   end
   
   it "should raise errors if given a regexp pattern match" do
@@ -291,12 +309,12 @@ describe "An SQLite dataset" do
 end
 
 describe "An SQLite dataset AS clause" do
-  it "should use a string literal for :col___alias" do
-    DB.literal(:c___a).must_equal "`c` AS 'a'"
+  it "should use a string literal for Sequel[:col].as(:alias)" do
+    DB.literal(Sequel[:c].as(:a)).must_equal "`c` AS 'a'"
   end
 
-  it "should use a string literal for :table__col___alias" do
-    DB.literal(:t__c___a).must_equal "`t`.`c` AS 'a'"
+  it "should use a string literal for Sequel[:table][:col].as(:alias)" do
+    DB.literal(Sequel[:t][:c].as(:a)).must_equal "`t`.`c` AS 'a'"
   end
 
   it "should use a string literal for :column.as(:alias)" do
@@ -304,11 +322,11 @@ describe "An SQLite dataset AS clause" do
   end
 
   it "should use a string literal in the SELECT clause" do
-    DB[:t].select(:c___a).sql.must_equal "SELECT `c` AS 'a' FROM `t`"
+    DB[:t].select(Sequel[:c].as(:a)).sql.must_equal "SELECT `c` AS 'a' FROM `t`"
   end
 
   it "should use a string literal in the FROM clause" do
-    DB[:t___a].sql.must_equal "SELECT * FROM `t` AS 'a'"
+    DB[Sequel[:t].as(:a)].sql.must_equal "SELECT * FROM `t` AS 'a'"
   end
 
   it "should use a string literal in the JOIN clause" do
@@ -329,6 +347,9 @@ describe "SQLite::Dataset#delete" do
     @d << {:name => 'def', :value => 4.56}
     @d << {:name => 'ghi', :value => 7.89}
   end
+  after do
+    DB.drop_table?(:items)
+  end
   
   it "should return the number of records affected when filtered" do
     @d.count.must_equal 3
@@ -439,10 +460,8 @@ describe "SQLite dataset" do
   end
   
   it "should have #explain work when identifier_output_method is modified" do
-    ds = DB[:test]
-    ds.identifier_output_method = :upcase
-    ds.explain.must_be_kind_of(String)
-  end
+    DB[:test].with_identifier_output_method(:upcase).explain.must_be_kind_of(String)
+  end if IDENTIFIER_MANGLING
 end
 
 describe "A SQLite database" do
@@ -454,7 +473,7 @@ describe "A SQLite database" do
     end
   end
   after do
-    @db.drop_table?(:test2)
+    @db.drop_table?(:test, :test2, :test3, :test3_backup0, :test3_backup1, :test3_backup2)
   end
 
   it "should support add_column operations" do
@@ -527,12 +546,10 @@ describe "A SQLite database" do
       @db.drop_column :test3, :value
 
       @db[:test].filter(:name => 'bar').delete
-      @db[:test3][:name => 'def'][:test_id].must_equal nil
+      @db[:test3][:name => 'def'][:test_id].must_be_nil
 
       @db[:test].filter(:name => 'foo').update(:id=>100)
       @db[:test3][:name => 'abc'][:test_id].must_equal 100
-
-      @db.drop_table? :test, :test3
     end
   end
 
@@ -566,9 +583,23 @@ describe "A SQLite database" do
     @db[:test3].first[:t].must_equal 'a'
     @db[:test3].delete
   end
+
+
+  it "should preserve autoincrement after table modification" do
+    @db.create_table!(:test2) do
+      primary_key :id
+      Integer :val, :null => false
+    end
+    @db.rename_column(:test2, :val, :value)
+
+    t = @db[:test2]
+    id1 = t.insert(:value=>1)
+    t.delete
+    id2 = t.insert(:value=>1)
+    id2.must_be :>, id1
+  end
   
   it "should handle quoted tables when dropping or renaming columns" do
-    @db.quote_identifiers = true
     table_name = "T T"
     @db.drop_table?(table_name)
     @db.create_table! table_name do
@@ -624,7 +655,6 @@ describe "A SQLite database" do
     @db[:test3_backup1].columns.must_equal [:k]
     @db[:test3_backup2].columns.must_equal [:l]
     @db.loggers.delete(l)
-    @db.drop_table?(:test3, :test3_backup0, :test3_backup1, :test3_backup2)
   end
   
   it "should support add_index" do
@@ -638,12 +668,12 @@ describe "A SQLite database" do
   end
 
   it "should keep applicable indexes when emulating schema methods" do
-    @db.create_table!(:a){Integer :a; Integer :b}
-    @db.add_index :a, :a
-    @db.add_index :a, :b
-    @db.add_index :a, [:b, :a]
-    @db.drop_column :a, :b
-    @db.indexes(:a).must_equal(:a_a_index=>{:unique=>false, :columns=>[:a]})
+    @db.create_table!(:test3){Integer :a; Integer :b}
+    @db.add_index :test3, :a
+    @db.add_index :test3, :b
+    @db.add_index :test3, [:b, :a]
+    @db.drop_column :test3, :b
+    @db.indexes(:test3).must_equal(:test3_a_index=>{:unique=>false, :columns=>[:a]})
   end
 
   it "should have support for various #transaction modes" do
@@ -663,7 +693,7 @@ describe "A SQLite database" do
       sqls.last.must_equal Sequel::Database::SQL_BEGIN
     end
 
-    @db.transaction_mode.must_equal nil
+    @db.transaction_mode.must_be_nil
     @db.transaction_mode = :immediate
     @db.transaction_mode.must_equal :immediate
     @db.transaction do
