@@ -3,7 +3,7 @@ require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 describe "Class Table Inheritance Plugin" do
   before(:all) do
     @db = DB
-    @db.instance_variable_set(:@schemas, {})
+    @db.instance_variable_get(:@schemas).clear
     @db.drop_table?(:staff, :executives, :managers, :employees)
     @db.create_table(:employees) do
       primary_key :id
@@ -70,27 +70,27 @@ describe "Class Table Inheritance Plugin" do
   end
   
   it "should lazily load columns in subclass tables" do
-    Employee[@i2][:manager_id].must_equal nil
+    Employee[@i2][:manager_id].must_be_nil
     Employee[@i2].manager_id.must_equal @i4
-    Employee[@i3][:num_staff].must_equal nil
+    Employee[@i3][:num_staff].must_be_nil
     Employee[@i3].num_staff.must_equal 7
-    Employee[@i4][:num_staff].must_equal nil
+    Employee[@i4][:num_staff].must_be_nil
     Employee[@i4].num_staff.must_equal 5
-    Employee[@i4][:num_managers].must_equal nil
+    Employee[@i4][:num_managers].must_be_nil
     Employee[@i4].num_managers.must_equal 6
-    Employee[@i5][:num_managers].must_equal nil
+    Employee[@i5][:num_managers].must_be_nil
     Employee[@i5].num_managers.must_equal 1
   end
   
   it "should eagerly load columns in subclass tables when retrieving multiple objects" do
     a = Employee.order(:id).all
-    a[1][:manager_id].must_equal nil
+    a[1][:manager_id].must_be_nil
     a[1].manager_id.must_equal @i4
-    a[2][:num_staff].must_equal nil
+    a[2][:num_staff].must_be_nil
     a[2].num_staff.must_equal 7
     a[3][:num_staff].must_equal 5 # eagerly loaded by previous call
     a[3].num_staff.must_equal 5
-    a[3][:num_managers].must_equal nil
+    a[3][:num_managers].must_be_nil
     a[3].num_managers.must_equal 6
     a[4][:num_managers].must_equal 1
     a[4].num_managers.must_equal 1
@@ -117,13 +117,13 @@ describe "Class Table Inheritance Plugin" do
     i = e.id
     e.staff_members_dataset.destroy
     e.destroy
-    @db[:executives][:id=>i].must_equal nil
-    @db[:managers][:id=>i].must_equal nil
-    @db[:employees][:id=>i].must_equal nil
+    @db[:executives][:id=>i].must_be_nil
+    @db[:managers][:id=>i].must_be_nil
+    @db[:employees][:id=>i].must_be_nil
   end
   
   it "should handle associations only defined in subclasses" do
-    Employee.filter(:employees__id=>@i2).all.first.manager.id.must_equal @i4
+    Employee.filter(Sequel[:employees][:id]=>@i2).all.first.manager.id.must_equal @i4
   end
 
   it "should insert rows into all tables" do
@@ -158,15 +158,15 @@ describe "Class Table Inheritance Plugin" do
   end
   
   it "should handle one_to_many relationships" do
-    Executive.first.staff_members.must_equal [Staff[@i2]]
+    Executive.first(:name=>'Ex').staff_members.must_equal [Staff[@i2]]
   end
   
   it "should handle eagerly loading one_to_many relationships" do
-    Executive.limit(1).eager(:staff_members).first.staff_members.must_equal [Staff[@i2]]
+    Executive.where(:name=>'Ex').eager(:staff_members).first.staff_members.must_equal [Staff[@i2]]
   end
   
   it "should handle eagerly graphing one_to_many relationships" do
-    es = Executive.limit(1).eager_graph(:staff_members).all
+    es = Executive.where(:name=>'Ex').eager_graph(:staff_members).all
     es.must_equal [Executive[@i4]]
     es.map{|x| x.staff_members}.must_equal [[Staff[@i2]]]
   end
@@ -175,7 +175,7 @@ end
 describe "Many Through Many Plugin" do
   before(:all) do
     @db = DB
-    @db.instance_variable_set(:@schemas, {})
+    @db.instance_variable_get(:@schemas).clear
     @db.drop_table?(:albums_artists, :albums, :artists)
     @db.create_table(:albums) do
       primary_key :id
@@ -248,10 +248,10 @@ describe "Many Through Many Plugin" do
     Artist.filter(:id=>@artist3.id).eager(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'B C'
     Artist.filter(:id=>@artist4.id).eager(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'B D'
     
-    Artist.filter(:artists__id=>@artist1.id).eager_graph(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'A D'
-    Artist.filter(:artists__id=>@artist2.id).eager_graph(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'A C'
-    Artist.filter(:artists__id=>@artist3.id).eager_graph(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'B C'
-    Artist.filter(:artists__id=>@artist4.id).eager_graph(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'B D'
+    Artist.filter(Sequel[:artists][:id]=>@artist1.id).eager_graph(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'A D'
+    Artist.filter(Sequel[:artists][:id]=>@artist2.id).eager_graph(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'A C'
+    Artist.filter(Sequel[:artists][:id]=>@artist3.id).eager_graph(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'B C'
+    Artist.filter(Sequel[:artists][:id]=>@artist4.id).eager_graph(:albums).all.map{|x| x.albums.map{|a| a.name}}.flatten.sort.must_equal %w'B D'
 
     Artist.filter(:albums=>@album1).all.map{|a| a.name}.sort.must_equal %w'1 2'
     Artist.filter(:albums=>@album2).all.map{|a| a.name}.sort.must_equal %w'3 4'
@@ -289,8 +289,8 @@ describe "Many Through Many Plugin" do
     c.exclude(:albums=>[@album1, @album3]).all.map{|a| a.name}.sort.must_equal %w'4'
     c.exclude(:albums=>[@album2, @album4]).all.map{|a| a.name}.sort.must_equal %w'2'
 
-    c.filter(:albums=>self_join(Album).filter(:albums__id=>[@album1.id, @album3.id])).all.map{|a| a.name}.sort.must_equal %w'1 2 3'
-    c.exclude(:albums=>self_join(Album).filter(:albums__id=>[@album1.id, @album3.id])).all.map{|a| a.name}.sort.must_equal %w'4'
+    c.filter(:albums=>self_join(Album).filter(Sequel[:albums][:id]=>[@album1.id, @album3.id])).all.map{|a| a.name}.sort.must_equal %w'1 2 3'
+    c.exclude(:albums=>self_join(Album).filter(Sequel[:albums][:id]=>[@album1.id, @album3.id])).all.map{|a| a.name}.sort.must_equal %w'4'
   end
 
   it "should handle typical case with 3 join tables" do
@@ -311,10 +311,10 @@ describe "Many Through Many Plugin" do
     Artist.filter(:id=>@artist3.id).eager(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'2 3 4'
     Artist.filter(:id=>@artist4.id).eager(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'1 3 4'
     
-    Artist.filter(:artists__id=>@artist1.id).eager_graph(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'1 2 4'
-    Artist.filter(:artists__id=>@artist2.id).eager_graph(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'1 2 3'
-    Artist.filter(:artists__id=>@artist3.id).eager_graph(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'2 3 4'
-    Artist.filter(:artists__id=>@artist4.id).eager_graph(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'1 3 4'
+    Artist.filter(Sequel[:artists][:id]=>@artist1.id).eager_graph(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'1 2 4'
+    Artist.filter(Sequel[:artists][:id]=>@artist2.id).eager_graph(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'1 2 3'
+    Artist.filter(Sequel[:artists][:id]=>@artist3.id).eager_graph(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'2 3 4'
+    Artist.filter(Sequel[:artists][:id]=>@artist4.id).eager_graph(:related_artists).all.map{|x| x.related_artists.map{|a| a.name}}.flatten.sort.must_equal %w'1 3 4'
 
     Artist.filter(:related_artists=>@artist1).all.map{|a| a.name}.sort.must_equal %w'1 2 4'
     Artist.filter(:related_artists=>@artist2).all.map{|a| a.name}.sort.must_equal %w'1 2 3'
@@ -346,8 +346,8 @@ describe "Many Through Many Plugin" do
     c.filter(:related_artists=>[@artist1, @artist4]).all.map{|a| a.name}.sort.must_equal %w'1 2 3 4'
     c.exclude(:related_artists=>[@artist1, @artist4]).all.map{|a| a.name}.sort.must_equal %w''
 
-    c.filter(:related_artists=>c.filter(:artists__id=>@artist1.id)).all.map{|a| a.name}.sort.must_equal %w'1 2 4'
-    c.exclude(:related_artists=>c.filter(:artists__id=>@artist1.id)).all.map{|a| a.name}.sort.must_equal %w'3'
+    c.filter(:related_artists=>c.filter(Sequel[:artists][:id]=>@artist1.id)).all.map{|a| a.name}.sort.must_equal %w'1 2 4'
+    c.exclude(:related_artists=>c.filter(Sequel[:artists][:id]=>@artist1.id)).all.map{|a| a.name}.sort.must_equal %w'3'
   end
 
   it "should handle extreme case with 5 join tables" do
@@ -377,10 +377,10 @@ describe "Many Through Many Plugin" do
     Artist.filter(:id=>@artist3.id).eager(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'A B D'
     Artist.filter(:id=>@artist4.id).eager(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'B D'
     
-    Artist.filter(:artists__id=>@artist1.id).eager_graph(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'A B C'
-    Artist.filter(:artists__id=>@artist2.id).eager_graph(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'A B C D'
-    Artist.filter(:artists__id=>@artist3.id).eager_graph(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'A B D'
-    Artist.filter(:artists__id=>@artist4.id).eager_graph(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'B D'
+    Artist.filter(Sequel[:artists][:id]=>@artist1.id).eager_graph(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'A B C'
+    Artist.filter(Sequel[:artists][:id]=>@artist2.id).eager_graph(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'A B C D'
+    Artist.filter(Sequel[:artists][:id]=>@artist3.id).eager_graph(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'A B D'
+    Artist.filter(Sequel[:artists][:id]=>@artist4.id).eager_graph(:related_albums).all.map{|x| x.related_albums.map{|a| a.name}}.flatten.sort.must_equal %w'B D'
 
     Artist.filter(:related_albums=>@album1).all.map{|a| a.name}.sort.must_equal %w'1 2 3'
     Artist.filter(:related_albums=>@album2).all.map{|a| a.name}.sort.must_equal %w'1 2 3 4'
@@ -418,8 +418,8 @@ describe "Many Through Many Plugin" do
     c.exclude(:related_albums=>[@album1, @album3]).all.map{|a| a.name}.sort.must_equal %w'4'
     c.exclude(:related_albums=>[@album2, @album4]).all.map{|a| a.name}.sort.must_equal %w''
 
-    c.filter(:related_albums=>self_join(Album).filter(:albums__id=>[@album1.id, @album3.id])).all.map{|a| a.name}.sort.must_equal %w'1 2 3'
-    c.exclude(:related_albums=>self_join(Album).filter(:albums__id=>[@album1.id, @album3.id])).all.map{|a| a.name}.sort.must_equal %w'4'
+    c.filter(:related_albums=>self_join(Album).filter(Sequel[:albums][:id]=>[@album1.id, @album3.id])).all.map{|a| a.name}.sort.must_equal %w'1 2 3'
+    c.exclude(:related_albums=>self_join(Album).filter(Sequel[:albums][:id]=>[@album1.id, @album3.id])).all.map{|a| a.name}.sort.must_equal %w'4'
   end
 end
 
@@ -470,7 +470,7 @@ end
 describe "Tactical Eager Loading Plugin" do
   before(:all) do
     @db = DB
-    @db.instance_variable_set(:@schemas, {})
+    @db.instance_variable_get(:@schemas).clear
     @db.drop_table?(:albums_artists)
     @db.create_table!(:artists) do
       primary_key :id
@@ -563,7 +563,7 @@ describe "Touch plugin" do
 
   it "should update the timestamp column when touching the record" do
     Album.plugin :touch
-    @album.updated_at.must_equal nil
+    @album.updated_at.must_be_nil
     @album.touch
     @album.updated_at.to_i.must_be_close_to Time.now.to_i, 2
   end
@@ -571,7 +571,7 @@ describe "Touch plugin" do
   cspecify "should update the timestamp column for many_to_one associated records when the record is updated or destroyed", [:do, :sqlite], [:jdbc, :sqlite], [:swift] do
     Album.many_to_one :artist
     Album.plugin :touch, :associations=>:artist
-    @artist.updated_at.must_equal nil
+    @artist.updated_at.must_be_nil
     @album.update(:name=>'B')
     ua = @artist.reload.updated_at
     if ua.is_a?(Time)
@@ -591,7 +591,7 @@ describe "Touch plugin" do
   cspecify "should update the timestamp column for one_to_many associated records when the record is updated", [:do, :sqlite], [:jdbc, :sqlite], [:swift] do
     Artist.one_to_many :albums
     Artist.plugin :touch, :associations=>:albums
-    @album.updated_at.must_equal nil
+    @album.updated_at.must_be_nil
     @artist.update(:name=>'B')
     ua = @album.reload.updated_at
     if ua.is_a?(Time)
@@ -605,7 +605,7 @@ describe "Touch plugin" do
     Artist.many_to_many :albums
     Artist.plugin :touch, :associations=>:albums
     @artist.add_album(@album)
-    @album.updated_at.must_equal nil
+    @album.updated_at.must_be_nil
     @artist.update(:name=>'B')
     ua = @album.reload.updated_at
     if ua.is_a?(Time)
@@ -717,7 +717,7 @@ describe "Composition plugin" do
 
   it "should return a composed object if the underlying columns have a value" do
     @e1.date.must_equal Date.civil(2010, 2, 15)
-    @e2.date.must_equal nil
+    @e2.date.must_be_nil
   end
 
   it "should decompose the object when saving the record" do
@@ -755,8 +755,8 @@ describe "RcteTree Plugin" do
       @aaab.children.must_equal []
       @aaaaa.children.must_equal []
       
-      @a.parent.must_equal nil
-      @b.parent.must_equal nil
+      @a.parent.must_be_nil
+      @b.parent.must_be_nil
       @aa.parent.must_equal @a
       @ab.parent.must_equal @a
       @ba.parent.must_equal @b
@@ -891,28 +891,28 @@ describe "RcteTree Plugin" do
     
     it "should populate all :parent associations when eagerly loading ancestors for a dataset" do
       nodes = @Node.filter(@Node.primary_key=>[@a.pk, @ba.pk, @aaa.pk, @aaaaa.pk]).order(:name).eager(:ancestors).all
-      nodes[0].associations.fetch(:parent, 1).must_equal nil
+      nodes[0].associations.fetch(:parent, 1).must_be_nil
       nodes[1].associations[:parent].must_equal @aa
       nodes[1].associations[:parent].associations[:parent].must_equal @a
-      nodes[1].associations[:parent].associations[:parent].associations.fetch(:parent, 1).must_equal nil
+      nodes[1].associations[:parent].associations[:parent].associations.fetch(:parent, 1).must_be_nil
       nodes[2].associations[:parent].must_equal @aaaa
       nodes[2].associations[:parent].associations[:parent].must_equal @aaa
       nodes[2].associations[:parent].associations[:parent].associations[:parent].must_equal @aa
       nodes[2].associations[:parent].associations[:parent].associations[:parent].associations[:parent].must_equal @a
-      nodes[2].associations[:parent].associations[:parent].associations[:parent].associations[:parent].associations.fetch(:parent, 1).must_equal nil
+      nodes[2].associations[:parent].associations[:parent].associations[:parent].associations[:parent].associations.fetch(:parent, 1).must_be_nil
       nodes[3].associations[:parent].must_equal @b
-      nodes[3].associations[:parent].associations.fetch(:parent, 1).must_equal nil
+      nodes[3].associations[:parent].associations.fetch(:parent, 1).must_be_nil
     end
     
     it "should populate all :parent associations when lazily loading ancestors" do
       @a.reload
       @a.ancestors
-      @a.associations[:parent].must_equal nil
+      @a.associations[:parent].must_be_nil
       
       @ba.reload
       @ba.ancestors
       @ba.associations[:parent].must_equal @b
-      @ba.associations[:parent].associations.fetch(:parent, 1).must_equal nil
+      @ba.associations[:parent].associations.fetch(:parent, 1).must_be_nil
       
       @ba.reload
       @aaaaa.ancestors
@@ -920,7 +920,7 @@ describe "RcteTree Plugin" do
       @aaaaa.associations[:parent].associations[:parent].must_equal @aaa
       @aaaaa.associations[:parent].associations[:parent].associations[:parent].must_equal @aa
       @aaaaa.associations[:parent].associations[:parent].associations[:parent].associations[:parent].must_equal @a
-      @aaaaa.associations[:parent].associations[:parent].associations[:parent].associations[:parent].associations.fetch(:parent, 1).must_equal nil
+      @aaaaa.associations[:parent].associations[:parent].associations[:parent].associations[:parent].associations.fetch(:parent, 1).must_be_nil
     end
   end
 
@@ -1302,6 +1302,11 @@ describe "AssociationPks plugin" do
     album.tag_pks.must_equal [@t1, @t2]
     album.save
     album_class.with_pk!(album.pk).tag_pks.sort.must_equal [@t1, @t2]
+
+    album.tag_pks = []
+    album.tag_pks.must_equal []
+    album.save
+    album_class.with_pk!(album.pk).tag_pks.sort.must_equal []
   end
 
   it "should handle :delay=>:all association option for existing instances" do
@@ -1313,6 +1318,11 @@ describe "AssociationPks plugin" do
     album.tag_pks.must_equal [@t1, @t2]
     album.save_changes
     album_class.with_pk!(album.pk).tag_pks.sort.must_equal [@t1, @t2]
+
+    album.tag_pks = []
+    album.tag_pks.must_equal []
+    album.save_changes
+    album_class.with_pk!(album.pk).tag_pks.sort.must_equal []
   end
 
   it "should set associated pks correctly for a one_to_many association" do
@@ -1330,6 +1340,9 @@ describe "AssociationPks plugin" do
     Artist[@ar1].album_pks = [@al1, @al2]
     Artist[@ar2].album_pks.must_equal [@al3]
     Album.order(:id).select_map(:artist_id).must_equal [@ar1, @ar1, @ar2]
+
+    Artist[@ar1].album_pks = []
+    Album.order(:id).select_map(:artist_id).must_equal [nil, nil, @ar2]
   end
 
   it "should set associated pks correctly for a many_to_many association" do
@@ -1369,6 +1382,9 @@ describe "AssociationPks plugin" do
     Album[@al1].vocalist_pks = [@v1, @v2]
     Album[@al2].vocalist_pks.must_equal [@v3]
     Vocalist.order(:first, :last).select_map(:album_id).must_equal [@al1, @al1, @al2]
+
+    Album[@al1].vocalist_pks = []
+    Vocalist.order(:first, :last).select_map(:album_id).must_equal [nil, nil, @al2]
   end
 
   it "should set associated right-side cpks correctly for a many_to_many association" do
@@ -1410,6 +1426,9 @@ describe "AssociationPks plugin" do
     Vocalist[@v1].instrument_pks = [@i1, @i2]
     Vocalist[@v2].instrument_pks.must_equal [@i3]
     Instrument.order(:id).select_map([:first, :last]).must_equal [@v1, @v1, @v2]
+
+    Vocalist[@v1].instrument_pks = []
+    Instrument.order(:id).select_map([:first, :last]).must_equal [[nil, nil], [nil, nil], @v2]
   end
 
   it "should set associated pks correctly with left-side cpks for a many_to_many association" do
@@ -1451,6 +1470,9 @@ describe "AssociationPks plugin" do
     Vocalist[@v1].hit_pks = [@h1, @h2]
     Vocalist[@v2].hit_pks.must_equal [@h3]
     Hit.order(:year, :week).select_map([:first, :last]).must_equal [@v1, @v1, @v2]
+
+    Vocalist[@v1].hit_pks = []
+    Hit.order(:year, :week).select_map([:first, :last]).must_equal [[nil, nil], [nil, nil], @v2]
   end
 
   it "should set associated right-side cpks correctly with left-side cpks for a many_to_many association" do
@@ -1506,12 +1528,12 @@ describe "List plugin without a scope" do
 
   it "should define prev and next" do
     i = @c[:name => "abc"]
-    i.prev.must_equal nil
+    i.prev.must_be_nil
     i = @c[:name => "def"]
     i.prev.must_equal @c[:name => "abc"]
     i.next.must_equal @c[:name => "hig"]
     i = @c[:name => "hig"]
-    i.next.must_equal nil
+    i.next.must_be_nil
   end
 
   it "should define move_to" do
@@ -1601,10 +1623,10 @@ describe "List plugin with a scope" do
     @c[:name => "P3"].prev(2).name.must_equal 'P1'
     @c[:name => "P2"].prev(-1).name.must_equal 'P3'
 
-    @c[:name => "Ps"].prev.must_equal nil
-    @c[:name => "Au"].next.must_equal nil
-    @c[:name => "P1"].prev.must_equal nil
-    @c[:name => "P3"].next.must_equal nil
+    @c[:name => "Ps"].prev.must_be_nil
+    @c[:name => "Au"].next.must_be_nil
+    @c[:name => "P1"].prev.must_be_nil
+    @c[:name => "P3"].next.must_be_nil
   end
 
   it "should define move_to" do
@@ -1811,31 +1833,31 @@ describe "Sequel::Plugins::PreparedStatements" do
   it "should work with looking up using Model.[]" do 
     @c[@foo.id].must_equal @foo
     @c[@bar.id].must_equal @bar
-    @c[0].must_equal nil
-    @c[nil].must_equal nil
+    @c[0].must_be_nil
+    @c[nil].must_be_nil
   end
 
   it "should work with looking up using Dataset#with_pk" do 
     @c.dataset.with_pk(@foo.id).must_equal @foo
     @c.dataset.with_pk(@bar.id).must_equal @bar
-    @c.dataset.with_pk(0).must_equal nil
-    @c.dataset.with_pk(nil).must_equal nil
+    @c.dataset.with_pk(0).must_be_nil
+    @c.dataset.with_pk(nil).must_be_nil
 
-    @c.dataset.filter(:i=>0).with_pk(@foo.id).must_equal nil
+    @c.dataset.filter(:i=>0).with_pk(@foo.id).must_be_nil
     @c.dataset.filter(:i=>10).with_pk(@foo.id).must_equal @foo
     @c.dataset.filter(:i=>20).with_pk(@bar.id).must_equal @bar
-    @c.dataset.filter(:i=>10).with_pk(nil).must_equal nil
+    @c.dataset.filter(:i=>10).with_pk(nil).must_be_nil
     @c.dataset.filter(:name=>'foo').with_pk(@foo.id).must_equal @foo
     @c.dataset.filter(:name=>'bar').with_pk(@bar.id).must_equal @bar
-    @c.dataset.filter(:name=>'baz').with_pk(@bar.id).must_equal nil
-    @c.dataset.filter(:name=>'bar').with_pk(nil).must_equal nil
+    @c.dataset.filter(:name=>'baz').with_pk(@bar.id).must_be_nil
+    @c.dataset.filter(:name=>'bar').with_pk(nil).must_be_nil
   end
 
   it "should work with Model#destroy" do 
     @foo.destroy
     @bar.destroy
-    @c[@foo.id].must_equal nil
-    @c[@bar.id].must_equal nil
+    @c[@foo.id].must_be_nil
+    @c[@bar.id].must_be_nil
   end
 
   it "should work with Model#update" do 
@@ -1871,7 +1893,7 @@ describe "Sequel::Plugins::PreparedStatements with schema changes" do
     @c = Class.new(Sequel::Model(@db[:ps_test]))
     @c.many_to_one :ps_test, :key=>:id, :class=>@c
     @c.one_to_many :ps_tests, :key=>:id, :class=>@c
-    @c.many_to_many :mps_tests, :left_key=>:id, :right_key=>:id, :class=>@c, :join_table=>:ps_test___x
+    @c.many_to_many :mps_tests, :left_key=>:id, :right_key=>:id, :class=>@c, :join_table=>Sequel[:ps_test].as(:x)
     @c.plugin :prepared_statements
     @c.plugin :prepared_statements_associations
   end
@@ -1930,8 +1952,8 @@ describe "Caching plugins" do
     it "should work with looking up using Model.[]" do 
       @Artist[1].must_be_same_as(@Artist[1])
       @Artist[:id=>1].must_equal @Artist[1]
-      @Artist[0].must_equal nil
-      @Artist[nil].must_equal nil
+      @Artist[0].must_be_nil
+      @Artist[nil].must_be_nil
     end
 
     it "should work with lookup up many_to_one associated objects" do 
@@ -1970,7 +1992,7 @@ end
 describe "Sequel::Plugins::ConstraintValidations" do
   before(:all) do
     @db = DB
-    @db.extension(:constraint_validations)
+    @db.extension(:constraint_validations) unless @db.frozen?
     @db.drop_table?(:sequel_constraint_validations)
     @db.create_constraint_validations_table
     @ds = @db[:cv_test]
@@ -1992,17 +2014,20 @@ describe "Sequel::Plugins::ConstraintValidations" do
       includes %w'abc def', :inc, opts.merge(:name=>:i)
       unique :uniq, opts.merge(:name=>:u)
       max_length 6, :minlen, opts.merge(:name=>:maxl2)
+      operator :<, 'm', :exactlen, opts.merge(:name=>:lt)
+      operator :>=, 5, :num, opts.merge(:name=>:gte)
     end
-    @valid_row = {:pre=>'a', :exactlen=>'12345', :minlen=>'12345', :maxlen=>'12345', :lenrange=>'1234', :lik=>'fooabc', :ilik=>'FooABC', :inc=>'abc', :uniq=>'u'}
+    @valid_row = {:pre=>'a', :exactlen=>'12345', :minlen=>'12345', :maxlen=>'12345', :lenrange=>'1234', :lik=>'fooabc', :ilik=>'FooABC', :inc=>'abc', :uniq=>'u', :num=>5}
     @violations = [
       [:pre, [nil, '', ' ']],
-      [:exactlen, [nil, '', '1234', '123456']],
+      [:exactlen, [nil, '', '1234', '123456', 'n1234']],
       [:minlen, [nil, '', '1234']],
       [:maxlen, [nil, '123456']],
       [:lenrange, [nil, '', '12', '123456']],
       [:lik, [nil, '', 'fo', 'fotabc', 'FOOABC']],
       [:ilik, [nil, '', 'fo', 'fotabc']],
       [:inc, [nil, '', 'ab', 'abcd']],
+      [:num, [nil, 3, 4]],
     ]
 
     if @regexp
@@ -2039,6 +2064,7 @@ describe "Sequel::Plugins::ConstraintValidations" do
     end
 
     it "should set up automatic validations inside the model" do 
+      skip if @db.frozen?
       c = Class.new(Sequel::Model(@ds))
       c.plugin :constraint_validations
       c.dataset.delete
@@ -2079,6 +2105,7 @@ describe "Sequel::Plugins::ConstraintValidations" do
           String :ilik
           String :inc
           String :uniq, :null=>false
+          Integer :num
           validate(&validate_block)
         end
       end
@@ -2124,6 +2151,7 @@ describe "Sequel::Plugins::ConstraintValidations" do
           if regexp
             add_column :form, String
           end
+          add_column :num, Integer
           validate(&validate_block)
         end
       end
@@ -2161,7 +2189,8 @@ describe "date_arithmetic extension" do
 
   before(:all) do
     @db = DB
-    @db.extension(:date_arithmetic)
+    @db.extension(:date_arithmetic) unless @db.frozen?
+    skip if @db.database_type == :sqlite && @db.frozen?
     if @db.database_type == :sqlite
       @db.use_timestamp_timezones = false
     end
@@ -2238,3 +2267,33 @@ describe "date_arithmetic extension" do
     @check.call(:date_sub, @dt, @h2, @s2)
   end
 end
+
+describe "string_agg extension" do
+  before(:all) do
+    @db = DB
+    @db.extension(:string_agg) unless @db.frozen?
+    @db.create_table!(:string_agg_test) do
+      Integer :id
+      String :s
+      Integer :o
+    end
+    @db[:string_agg_test].import([:id, :s, :o], [[1, 'a', 3], [1, 'a', 3], [1, 'b', 5], [1, 'c', 4], [2, 'aa', 2], [2, 'bb', 1]])
+    @ds = @db[:string_agg_test].select_group(:id).order(:id)
+  end
+  after(:all) do
+    @db.drop_table?(:string_agg_test)
+  end
+
+  cspecify "should have string_agg return aggregated concatenation", :mssql, :sqlite, :derby do
+    h = @ds.select_append(Sequel.string_agg(:s).as(:v)).to_hash(:id, :v)
+    h[1].must_match(/\A[abc],[abc],[abc],[abc]\z/)
+    h[2].must_match(/\A(aa|bb),(aa|bb)\z/)
+
+    @ds.select_append(Sequel.string_agg(:s).order(:o).as(:v)).map([:id, :v]).must_equal [[1, 'a,a,c,b'], [2, 'bb,aa']]
+    @ds.select_append(Sequel.string_agg(:s, '-').order(:o).as(:v)).map([:id, :v]).must_equal [[1, 'a-a-c-b'], [2, 'bb-aa']]
+  end
+
+  cspecify "should have string_agg return aggregated concatenation for distinct values", :mssql, :sqlite, :oracle, :db2, :derby do
+    @ds.select_group(:id).select_append(Sequel.string_agg(:s).order(:s).distinct.as(:v)).map([:id, :v]).must_equal [[1, 'a,b,c'], [2, 'aa,bb']]
+  end
+end
