@@ -55,14 +55,14 @@ describe "Prepared Statements and Bound Arguments" do
   end
 
   it "should support placeholder literal strings with call" do
-    @ds.filter("numb = ?", :$n).call(:select, :n=>10).must_equal [{:id=>1, :numb=>10}]
+    @ds.filter(Sequel.lit("numb = ?", :$n)).call(:select, :n=>10).must_equal [{:id=>1, :numb=>10}]
   end
 
   it "should support named placeholder literal strings and handle multiple named placeholders correctly with call" do
-    @ds.filter("numb = :n", :n=>:$n).call(:select, :n=>10).must_equal [{:id=>1, :numb=>10}]
+    @ds.filter(Sequel.lit("numb = :n", :n=>:$n)).call(:select, :n=>10).must_equal [{:id=>1, :numb=>10}]
     @ds.insert(:numb=>20)
     @ds.insert(:numb=>30)
-    @ds.filter("numb > :n1 AND numb < :n2 AND numb = :n3", :n3=>:$n3, :n2=>:$n2, :n1=>:$n1).call(:select, :n3=>20, :n2=>30, :n1=>10).must_equal [{:id=>2, :numb=>20}]
+    @ds.filter(Sequel.lit("numb > :n1 AND numb < :n2 AND numb = :n3", :n3=>:$n3, :n2=>:$n2, :n1=>:$n1)).call(:select, :n3=>20, :n2=>30, :n1=>10).must_equal [{:id=>2, :numb=>20}]
   end
 
   it "should support datasets with static sql and placeholders with call" do
@@ -78,7 +78,7 @@ describe "Prepared Statements and Bound Arguments" do
   end
 
   it "should support subselects with literal strings with call" do
-    @ds.filter(:id=>:$i, :numb=>@ds.select(:numb).filter("numb = ?", :$n)).call(:select, :n=>10, :i=>1).must_equal [{:id=>1, :numb=>10}]
+    @ds.filter(:id=>:$i, :numb=>@ds.select(:numb).filter(Sequel.lit("numb = ?", :$n))).call(:select, :n=>10, :i=>1).must_equal [{:id=>1, :numb=>10}]
   end
 
   it "should support subselects with static sql and placeholders with call" do
@@ -186,14 +186,14 @@ describe "Prepared Statements and Bound Arguments" do
   end
 
   it "should support placeholder literal strings with prepare" do
-    @ds.filter("numb = ?", :$n).prepare(:select, :seq_select).call(:n=>10).must_equal [{:id=>1, :numb=>10}]
+    @ds.filter(Sequel.lit("numb = ?", :$n)).prepare(:select, :seq_select).call(:n=>10).must_equal [{:id=>1, :numb=>10}]
   end
 
   it "should support named placeholder literal strings and handle multiple named placeholders correctly with prepare" do
-    @ds.filter("numb = :n", :n=>:$n).prepare(:select, :seq_select).call(:n=>10).must_equal [{:id=>1, :numb=>10}]
+    @ds.filter(Sequel.lit("numb = :n", :n=>:$n)).prepare(:select, :seq_select).call(:n=>10).must_equal [{:id=>1, :numb=>10}]
     @ds.insert(:numb=>20)
     @ds.insert(:numb=>30)
-    @ds.filter("numb > :n1 AND numb < :n2 AND numb = :n3", :n3=>:$n3, :n2=>:$n2, :n1=>:$n1).call(:select, :n3=>20, :n2=>30, :n1=>10).must_equal [{:id=>2, :numb=>20}]
+    @ds.filter(Sequel.lit("numb > :n1 AND numb < :n2 AND numb = :n3", :n3=>:$n3, :n2=>:$n2, :n1=>:$n1)).call(:select, :n3=>20, :n2=>30, :n1=>10).must_equal [{:id=>2, :numb=>20}]
   end
 
   it "should support datasets with static sql and placeholders with prepare" do
@@ -209,7 +209,7 @@ describe "Prepared Statements and Bound Arguments" do
   end
 
   it "should support subselects with literal strings with prepare" do
-    @ds.filter(:id=>:$i, :numb=>@ds.select(:numb).filter("numb = ?", :$n)).prepare(:select, :seq_select).call(:n=>10, :i=>1).must_equal [{:id=>1, :numb=>10}]
+    @ds.filter(:id=>:$i, :numb=>@ds.select(:numb).filter(Sequel.lit("numb = ?", :$n))).prepare(:select, :seq_select).call(:n=>10, :i=>1).must_equal [{:id=>1, :numb=>10}]
   end
 
   it "should support subselects with static sql and placeholders with prepare" do
@@ -371,7 +371,7 @@ describe "Bound Argument Types" do
   cspecify "should handle blob type with nil values", [:oracle], [:tinytds], [:jdbc, :mssql] do
     @ds.delete
     @ds.prepare(:insert, :ps_blob, {:file=>:$x}).call(:x=>nil)
-    @ds.get(:file).must_equal nil
+    @ds.get(:file).must_be_nil
   end
 
   cspecify "should handle blob type with embedded zeros", [:odbc] do
@@ -412,10 +412,10 @@ describe "Dataset#unbind" do
   it "should unbind values assigned to equality and inequality statements" do
     @ct[Integer, 10]
     @u[@ds.filter(:c=>10)].must_equal(:c=>10)
-    @u[@ds.exclude(:c=>10)].must_equal nil
-    @u[@ds.filter{c < 10}].must_equal nil
+    @u[@ds.exclude(:c=>10)].must_be_nil
+    @u[@ds.filter{c < 10}].must_be_nil
     @u[@ds.filter{c <= 10}].must_equal(:c=>10)
-    @u[@ds.filter{c > 10}].must_equal nil
+    @u[@ds.filter{c > 10}].must_be_nil
     @u[@ds.filter{c >= 10}].must_equal(:c=>10)
   end
 
@@ -444,7 +444,7 @@ describe "Dataset#unbind" do
 
   it "should handle QualifiedIdentifiers" do
     @ct[Integer, 10]
-    @u[@ds.filter{items__c > 1}].must_equal(:c=>10)
+    @u[@ds.filter{items[:c] > 1}].must_equal(:c=>10)
   end
 
   cspecify "should handle deep nesting", :h2 do
@@ -456,12 +456,12 @@ describe "Dataset#unbind" do
     end
     @ds.insert(:a=>2, :b=>0, :c=>3, :d=>5)
     @u[@ds.filter{a > 1}.and{b < 2}.or(:c=>3).and(Sequel.case({~Sequel.expr(:d=>4)=>1}, 0) => 1)].must_equal(:a=>2, :b=>0, :c=>3, :d=>5)
-    @u[@ds.filter{a > 1}.and{b < 2}.or(:c=>3).and(Sequel.case({~Sequel.expr(:d=>5)=>1}, 0) => 1)].must_equal nil
+    @u[@ds.filter{a > 1}.and{b < 2}.or(:c=>3).and(Sequel.case({~Sequel.expr(:d=>5)=>1}, 0) => 1)].must_be_nil
   end
 
   it "should handle case where the same variable has the same value in multiple places " do
     @ct[Integer, 1]
     @u[@ds.filter{c > 1}.or{c < 1}.invert].must_equal(:c=>1)
-    @u[@ds.filter{c > 1}.or{c < 1}].must_equal nil
+    @u[@ds.filter{c > 1}.or{c < 1}].must_be_nil
   end
 end    
