@@ -40,7 +40,7 @@ describe "Database transactions" do
     @db.transaction do
       @d << {:name => 'abc', :value => 1}
       raise Sequel::Rollback
-    end.must_equal nil
+    end.must_be_nil
 
     proc do
       @db.transaction(:rollback=>:reraise) do
@@ -84,6 +84,38 @@ describe "Database transactions" do
   end 
   
   if DB.supports_savepoints?
+    it "should handle table_exists? failures inside transactions" do
+      @db.transaction do
+        @d << {:name => '1'}
+        @db.table_exists?(:asadf098asd9asd98sa).must_equal false
+        @d << {:name => '2'}
+      end
+      @d.select_order_map(:name).must_equal %w'1 2'
+    end
+
+    it "should handle :rollback=>:always inside transactions" do
+      @db.transaction do
+        @db.transaction(:rollback=>:always) do
+          @d << {:name => 'abc', :value => 1}
+          2
+        end.must_equal 2
+      end
+      @d.select_order_map(:value).must_equal []
+    end
+
+    it "should handle table_exists? failures inside savepoints" do
+      @db.transaction do
+        @d << {:name => '1'}
+        @db.transaction(:savepoint=>true) do
+          @d << {:name => '2'}
+          @db.table_exists?(:asadf098asd9asd98sa).must_equal false
+          @d << {:name => '3'}
+        end
+        @d << {:name => '4'}
+      end
+      @d.select_order_map(:name).must_equal %w'1 2 3 4'
+    end
+
     it "should support nested transactions through savepoints using the savepoint option" do
       @db.transaction do
         @d << {:name => '1'}
@@ -134,17 +166,17 @@ describe "Database transactions" do
   end
 
   it "should handle returning inside of the block by committing" do
-    def @db.ret_commit
-      transaction do
-        self[:items] << {:name => 'abc'}
+    def ret_commit
+      @db.transaction do
+        @db[:items] << {:name => 'abc'}
         return
       end
     end
 
     @d.count.must_equal 0
-    @db.ret_commit
+    ret_commit
     @d.count.must_equal 1
-    @db.ret_commit
+    ret_commit
     @d.count.must_equal 2
     proc do
       @db.transaction do
@@ -203,30 +235,30 @@ describe "Database transactions" do
   it "should support after_rollback outside transactions" do
     c = nil
     @db.after_rollback{c = 1}
-    c.must_equal nil
+    c.must_be_nil
   end
 
   it "should support after_commit inside transactions" do
     c = nil
-    @db.transaction{@db.after_commit{c = 1}; c.must_equal nil}
+    @db.transaction{@db.after_commit{c = 1}; c.must_be_nil}
     c.must_equal 1
   end
 
   it "should support after_rollback inside transactions" do
     c = nil
-    @db.transaction{@db.after_rollback{c = 1}; c.must_equal nil}
-    c.must_equal nil
+    @db.transaction{@db.after_rollback{c = 1}; c.must_be_nil}
+    c.must_be_nil
   end
 
   it "should not call after_commit if the transaction rolls back" do
     c = nil
-    @db.transaction{@db.after_commit{c = 1}; c.must_equal nil; raise Sequel::Rollback}
-    c.must_equal nil
+    @db.transaction{@db.after_commit{c = 1}; c.must_be_nil; raise Sequel::Rollback}
+    c.must_be_nil
   end
 
   it "should call after_rollback if the transaction rolls back" do
     c = nil
-    @db.transaction{@db.after_rollback{c = 1}; c.must_equal nil; raise Sequel::Rollback}
+    @db.transaction{@db.after_rollback{c = 1}; c.must_be_nil; raise Sequel::Rollback}
     c.must_equal 1
   end
 
@@ -244,26 +276,26 @@ describe "Database transactions" do
 
   it "should support after_commit inside nested transactions" do
     c = nil
-    @db.transaction{@db.transaction{@db.after_commit{c = 1}}; c.must_equal nil}
+    @db.transaction{@db.transaction{@db.after_commit{c = 1}}; c.must_be_nil}
     c.must_equal 1
   end
 
   it "should support after_rollback inside nested transactions" do
     c = nil
-    @db.transaction{@db.transaction{@db.after_rollback{c = 1}}; c.must_equal nil; raise Sequel::Rollback}
+    @db.transaction{@db.transaction{@db.after_rollback{c = 1}}; c.must_be_nil; raise Sequel::Rollback}
     c.must_equal 1
   end
 
   if DB.supports_savepoints?
     it "should support after_commit inside savepoints" do
       c = nil
-      @db.transaction{@db.transaction(:savepoint=>true){@db.after_commit{c = 1}}; c.must_equal nil}
+      @db.transaction{@db.transaction(:savepoint=>true){@db.after_commit{c = 1}}; c.must_be_nil}
       c.must_equal 1
     end
 
     it "should support after_rollback inside savepoints" do
       c = nil
-      @db.transaction{@db.transaction(:savepoint=>true){@db.after_rollback{c = 1}}; c.must_equal nil; raise Sequel::Rollback}
+      @db.transaction{@db.transaction(:savepoint=>true){@db.after_rollback{c = 1}}; c.must_be_nil; raise Sequel::Rollback}
       c.must_equal 1
     end
   end
@@ -283,7 +315,7 @@ describe "Database transactions" do
   end
 end
 
-if (! defined?(RUBY_ENGINE) or RUBY_ENGINE == 'ruby' or (RUBY_ENGINE == 'rbx' && !Sequel.guarded?([:do, :sqlite], [:tinytds, :mssql]))) and RUBY_VERSION < '1.9'
+if (! defined?(RUBY_ENGINE) or RUBY_ENGINE == 'ruby') and RUBY_VERSION < '1.9'
   describe "Database transactions and Thread#kill" do
     before do
       @db = DB
