@@ -160,7 +160,7 @@ describe Sequel::Model, ".def_dataset_method" do
     @c.def_dataset_method(:return_3, :return_4)
     proc{@c.return_3}.must_raise(NoMethodError)
     proc{@c.return_4}.must_raise(NoMethodError)
-    @c.dataset.instance_eval do
+    @c.dataset = @c.dataset.with_extend do
       def return_3; 3; end
       def return_4; 4; end
     end
@@ -279,7 +279,122 @@ describe Sequel::Model, ".dataset_module" do
     @c.where(:foo).released.sql.must_equal 'SELECT * FROM items WHERE (foo AND released)'
   end
 
-  it "should raise error if called with both an argument and ablock" do
+  it "should have dataset_module support a where method" do
+    @c.dataset_module{where :released, :released}
+    @c.released.sql.must_equal 'SELECT * FROM items WHERE released'
+    @c.where(:foo).released.sql.must_equal 'SELECT * FROM items WHERE (foo AND released)'
+  end
+
+  it "should have dataset_module support a having method" do
+    @c.dataset_module{having(:released){released}}
+    @c.released.sql.must_equal 'SELECT * FROM items HAVING released'
+    @c.where(:foo).released.sql.must_equal 'SELECT * FROM items WHERE foo HAVING released'
+  end
+
+  it "should have dataset_module support an exclude method" do
+    @c.dataset_module{exclude :released, :released}
+    @c.released.sql.must_equal 'SELECT * FROM items WHERE NOT released'
+    @c.where(:foo).released.sql.must_equal 'SELECT * FROM items WHERE (foo AND NOT released)'
+  end
+
+  it "should have dataset_module support an exclude_having method" do
+    @c.dataset_module{exclude_having :released, :released}
+    @c.released.sql.must_equal 'SELECT * FROM items HAVING NOT released'
+    @c.where(:foo).released.sql.must_equal 'SELECT * FROM items WHERE foo HAVING NOT released'
+  end
+
+  it "should have dataset_module support a distinct method" do
+    @c.dataset = @c.dataset.with_extend{def supports_distinct_on?; true end}
+    @c.dataset_module{distinct :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT DISTINCT ON (baz) * FROM items'
+    @c.where(:bar).foo.sql.must_equal 'SELECT DISTINCT ON (baz) * FROM items WHERE bar'
+  end
+
+  it "should have dataset_module support a grep method" do
+    @c.dataset_module{grep :foo, :baz, 'quux%'}
+    @c.foo.sql.must_equal 'SELECT * FROM items WHERE ((baz LIKE \'quux%\' ESCAPE \'\\\'))'
+    @c.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE (bar AND ((baz LIKE \'quux%\' ESCAPE \'\\\')))'
+  end
+
+  it "should have dataset_module support a group method" do
+    @c.dataset_module{group :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT * FROM items GROUP BY baz'
+    @c.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE bar GROUP BY baz'
+  end
+
+  it "should have dataset_module support a group_and_count method" do
+    @c.dataset_module{group_and_count :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT baz, count(*) AS count FROM items GROUP BY baz'
+    @c.where(:bar).foo.sql.must_equal 'SELECT baz, count(*) AS count FROM items WHERE bar GROUP BY baz'
+  end
+
+  it "should have dataset_module support a group_append method" do
+    @c.dataset_module{group_append :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT * FROM items GROUP BY baz'
+    @c.group(:bar).foo.sql.must_equal 'SELECT * FROM items GROUP BY bar, baz'
+  end
+
+  it "should have dataset_module support a limit method" do
+    @c.dataset_module{limit :foo, 1}
+    @c.foo.sql.must_equal 'SELECT * FROM items LIMIT 1'
+    @c.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE bar LIMIT 1'
+  end
+
+  it "should have dataset_module support a offset method" do
+    @c.dataset_module{offset :foo, 1}
+    @c.foo.sql.must_equal 'SELECT * FROM items OFFSET 1'
+    @c.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE bar OFFSET 1'
+  end
+
+  it "should have dataset_module support a order method" do
+    @c.dataset_module{order(:foo){:baz}}
+    @c.foo.sql.must_equal 'SELECT * FROM items ORDER BY baz'
+    @c.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE bar ORDER BY baz'
+  end
+
+  it "should have dataset_module support a order_append method" do
+    @c.dataset_module{order_append :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT * FROM items ORDER BY baz'
+    @c.order(:bar).foo.sql.must_equal 'SELECT * FROM items ORDER BY bar, baz'
+  end
+
+  it "should have dataset_module support a order_prepend method" do
+    @c.dataset_module{order_prepend :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT * FROM items ORDER BY baz'
+    @c.order(:bar).foo.sql.must_equal 'SELECT * FROM items ORDER BY baz, bar'
+  end
+
+  it "should have dataset_module support a select method" do
+    @c.dataset_module{select :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT baz FROM items'
+    @c.where(:bar).foo.sql.must_equal 'SELECT baz FROM items WHERE bar'
+  end
+
+  it "should have dataset_module support a select_all method" do
+    @c.dataset_module{select_all :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT baz.* FROM items'
+    @c.where(:bar).foo.sql.must_equal 'SELECT baz.* FROM items WHERE bar'
+  end
+
+  it "should have dataset_module support a select_append method" do
+    @c.dataset_module{select_append :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT *, baz FROM items'
+    @c.where(:bar).foo.sql.must_equal 'SELECT *, baz FROM items WHERE bar'
+  end
+
+  it "should have dataset_module support a select_group method" do
+    @c.dataset_module{select_group :foo, :baz}
+    @c.foo.sql.must_equal 'SELECT baz FROM items GROUP BY baz'
+    @c.where(:bar).foo.sql.must_equal 'SELECT baz FROM items WHERE bar GROUP BY baz'
+  end
+
+  it "should have dataset_module support a server method" do
+    @c.dataset_module{server :foo, :baz}
+    @c.foo.opts[:server].must_equal :baz
+    @c.where(:bar).foo.opts[:server].must_equal :baz
+  end
+
+  it "should raise error if called with both an argument and a block" do
     proc{@c.dataset_module(Module.new{def return_3() 3 end}){}}.must_raise(Sequel::Error)
   end
 end
@@ -333,7 +448,7 @@ describe "Model.primary_key" do
   
   it "should use nil for no primary key" do
     @c.no_primary_key
-    @c.primary_key.must_equal nil
+    @c.primary_key.must_be_nil
   end
 end
 
@@ -392,7 +507,7 @@ describe "Model.db" do
     Sequel::DATABASES.clear
   end
   after do
-    Sequel::Model.instance_variable_get(:@db).must_equal nil
+    Sequel::Model.instance_variable_get(:@db).must_be_nil
     Sequel::DATABASES.replace(@databases)
     Sequel::Model.db = @model_db
   end
@@ -470,7 +585,7 @@ describe Sequel::Model, ".(allowed|restricted)_columns " do
   end
   
   it "should set the allowed columns correctly" do
-    @c.allowed_columns.must_equal nil
+    @c.allowed_columns.must_be_nil
     @c.set_allowed_columns :x
     @c.allowed_columns.must_equal [:x]
     @c.set_allowed_columns :x, :y
@@ -484,11 +599,11 @@ describe Sequel::Model, ".(allowed|restricted)_columns " do
     i.set(:x => 4, :y => 5, :z => 6)
     i.values.must_equal(:x => 4, :y => 5)
 
-    @c.instance_dataset._fetch = @c.dataset._fetch = {:x => 7}
+    @c.dataset = @c.dataset.with_fetch(:x => 7)
     i = @c.new
     i.update(:x => 7, :z => 9)
     i.values.must_equal(:x => 7)
-    DB.sqls.must_equal ["INSERT INTO blahblah (x) VALUES (7)", "SELECT * FROM blahblah WHERE (id = 10) LIMIT 1"]
+    DB.sqls.must_equal ["INSERT INTO blahblah (x) VALUES (7)", "SELECT * FROM blahblah WHERE id = 10"]
   end
 end
 
@@ -563,10 +678,8 @@ end
 
 describe Sequel::Model, ".require_modification" do
   before do
-    @ds1 = DB[:items]
-    def @ds1.provides_accurate_rows_matched?() false end
-    @ds2 = DB[:items]
-    def @ds2.provides_accurate_rows_matched?() true end
+    @ds1 = DB[:items].with_extend{def provides_accurate_rows_matched?; false end}
+    @ds2 = DB[:items].with_extend{def provides_accurate_rows_matched?; true end}
   end
   after do
     Sequel::Model.require_modification = nil
@@ -591,89 +704,74 @@ end
 describe Sequel::Model, ".[] optimization" do
   before do
     @db = Sequel.mock
-    @db.quote_identifiers = true
     def @db.schema(*) [[:id, {:primary_key=>true}]] end
     def @db.supports_schema_parsing?() true end
     @c = Class.new(Sequel::Model(@db))
+    @ds = @db.dataset.with_quote_identifiers(true)
   end
 
   it "should set simple_pk to the literalized primary key column name if a single primary key" do
     @c.set_primary_key :id
-    @c.simple_pk.must_equal '"id"'
+    @c.simple_pk.must_equal 'id'
     @c.set_primary_key :b
-    @c.simple_pk.must_equal '"b"'
+    @c.simple_pk.must_equal 'b'
     @c.set_primary_key Sequel.identifier(:b__a)
-    @c.simple_pk.must_equal '"b__a"'
+    @c.simple_pk.must_equal 'b__a'
   end
 
   it "should have simple_pk be blank if compound or no primary key" do
     @c.no_primary_key
-    @c.simple_pk.must_equal nil
+    @c.simple_pk.must_be_nil
     @c.set_primary_key [:b, :a]
-    @c.simple_pk.must_equal nil
+    @c.simple_pk.must_be_nil
   end
 
   it "should have simple table set if passed a Symbol to set_dataset" do
     @c.set_dataset :a
-    @c.simple_table.must_equal '"a"'
+    @c.simple_table.must_equal 'a'
     @c.set_dataset :b
-    @c.simple_table.must_equal '"b"'
+    @c.simple_table.must_equal 'b'
     @c.set_dataset :b__a
-    @c.simple_table.must_equal '"b"."a"'
+    @c.simple_table.must_equal 'b.a'
   end
 
   it "should have simple_table set if passed a simple select all dataset to set_dataset" do
-    @c.set_dataset @db[:a]
+    @c.set_dataset @ds.from(:a)
     @c.simple_table.must_equal '"a"'
-    @c.set_dataset @db[:b]
+    @c.set_dataset @ds.from(:b)
     @c.simple_table.must_equal '"b"'
-    @c.set_dataset @db[:b__a]
+    @c.set_dataset @ds.from(:b__a)
     @c.simple_table.must_equal '"b"."a"'
   end
 
-  it "should have simple_pk and simple_table respect dataset's identifier input methods" do
-    ds = @db[:ab]
-    ds.identifier_input_method = :reverse
-    @c.set_dataset ds
-    @c.simple_table.must_equal '"ba"'
-    @c.set_primary_key :cd
-    @c.simple_pk.must_equal '"dc"'
-
-    @c.set_dataset ds.from(:ef__gh)
-    @c.simple_table.must_equal '"fe"."hg"'
-  end
-
   it "should have simple_table = nil if passed a non-simple select all dataset to set_dataset" do
     @c.set_dataset @c.db[:a].filter(:active)
-    @c.simple_table.must_equal nil
+    @c.simple_table.must_be_nil
   end
 
   it "should have simple_table inherit superclass's setting" do
-    Class.new(@c).simple_table.must_equal nil
+    Class.new(@c).simple_table.must_be_nil
     @c.set_dataset :a
-    Class.new(@c).simple_table.must_equal '"a"'
+    Class.new(@c).simple_table.must_equal 'a'
   end
 
   it "should use Dataset#with_sql if simple_table and simple_pk are true" do
-    @c.set_dataset :a
-    @c.instance_dataset._fetch = @c.dataset._fetch = {:id => 1}
+    @c.set_dataset @db[:a].with_fetch(:id=>1)
     @c[1].must_equal @c.load(:id=>1)
-    @db.sqls.must_equal ['SELECT * FROM "a" WHERE "id" = 1']
+    @db.sqls.must_equal ['SELECT * FROM a WHERE id = 1']
   end
 
   it "should not use Dataset#with_sql if either simple_table or simple_pk is nil" do
-    @c.set_dataset @db[:a].filter(:active)
-    @c.dataset._fetch = {:id => 1}
+    @c.set_dataset @db[:a].where(:active).with_fetch(:id=>1)
     @c[1].must_equal @c.load(:id=>1)
-    @db.sqls.must_equal ['SELECT * FROM "a" WHERE ("active" AND ("id" = 1)) LIMIT 1']
+    @db.sqls.must_equal ['SELECT * FROM a WHERE (active AND (id = 1)) LIMIT 1']
   end
 end
 
 describe "Model datasets #with_pk with #with_pk!" do
   before do
     @c = Class.new(Sequel::Model(:a))
-    @ds = @c.dataset
-    @ds._fetch = {:id=>1}
+    @ds = @c.dataset = @c.dataset.with_fetch(:id=>1)
     DB.reset
   end
 
@@ -691,6 +789,14 @@ describe "Model datasets #with_pk with #with_pk!" do
     DB.sqls.must_equal ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
   end
 
+  it "should work when called repeatedly on a frozen dataset" do
+    @ds.freeze
+    5.times do
+      @ds.with_pk(1).must_equal @c.load(:id=>1)
+      DB.sqls.must_equal ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
+    end
+  end
+
   it "should handle existing filters" do
     @ds.filter(:a=>2).with_pk(1)
     DB.sqls.must_equal ["SELECT * FROM a WHERE ((a = 2) AND (a.id = 1)) LIMIT 1"]
@@ -720,17 +826,29 @@ describe "Model datasets #with_pk with #with_pk!" do
     sqls.must_equal []
   end
 
+  it "should work with composite primary keys when called repeatedly on a frozen dataset with" do
+    @c.set_primary_key [:id1, :id2]
+    @ds.freeze
+    5.times do
+      @ds.with_pk([1,2])
+      sqls = DB.sqls
+      ["SELECT * FROM a WHERE ((a.id1 = 1) AND (a.id2 = 2)) LIMIT 1",
+      "SELECT * FROM a WHERE ((a.id2 = 2) AND (a.id1 = 1)) LIMIT 1"].must_include(sqls.pop)
+      sqls.must_equal []
+    end
+  end
+
   it "should have with_pk return nil and with_pk! raise if no rows match" do
-    @ds._fetch = []
-    @ds.with_pk(1).must_equal nil
+    @ds = @ds.with_fetch([])
+    @ds.with_pk(1).must_be_nil
     DB.sqls.must_equal ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
     proc{@ds.with_pk!(1)}.must_raise(Sequel::NoMatchingRow)
     DB.sqls.must_equal ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
   end
 
   it "should have with_pk return nil and with_pk! raise if no rows match when calling the class method" do
-    @ds._fetch = []
-    @c.with_pk(1).must_equal nil
+    @c.dataset = @c.dataset.with_fetch([])
+    @c.with_pk(1).must_be_nil
     DB.sqls.must_equal ["SELECT * FROM a WHERE id = 1"]
     proc{@c.with_pk!(1)}.must_raise(Sequel::NoMatchingRow)
     DB.sqls.must_equal ["SELECT * FROM a WHERE id = 1"]
@@ -745,6 +863,14 @@ describe "Model datasets #with_pk with #with_pk!" do
     @ds['foo'].must_equal @c.load(:id=>1)
     DB.sqls.must_equal ["SELECT * FROM a WHERE (foo) LIMIT 1"]
   end
+
+  it "should raise Error if called on a dataset with no primary key" do
+    @c.no_primary_key
+    @ds.freeze
+    5.times do
+      proc{@ds.with_pk(1)}.must_raise Sequel::Error
+    end
+  end
 end
 
 describe "Model::include" do
